@@ -2,6 +2,7 @@ import datetime
 import dns.resolver
 import json
 import logging
+import os
 import re
 import xmltodict
 from urllib.parse import urlparse
@@ -18,34 +19,52 @@ def convert_xml_to_dict(xml_string, json_output=False):
         dict: The XML string converted to a dictionary.
     """
     if json_output:
+        logging.debug("Converting XML to JSON object...")
         return json.dumps(xmltodict.parse(xml_string))
 
     else:
+        logging.debug("Converting XML to dictionary...")
         return xmltodict.parse(xml_string)
 
 
-def initialize_logger(file_handler_path=None):
+def initialize_logger(logger_level="INFO", file_handler_path=None):
     """Initializes a logger with a stream handler and an optional file handler.
 
     Args:
+        logger_level (str, optional): The level of the logger. Defaults to "INFO".
+                                      Can be "DEBUG", "INFO", "WARNING", "ERROR", or "CRITICAL".
         file_handler_path (str, optional): The path to save the log file if a file handler is desired. Defaults to None.
     """
     # set up logger
     today = datetime.datetime.now().strftime("%Y_%m_%d")
     logger = logging.getLogger()  # root logger
 
+    # Mapping of string levels to logging constants
+    level_mapping = {
+        "DEBUG": logging.DEBUG,
+        "INFO": logging.INFO,
+        "WARNING": logging.WARNING,
+        "ERROR": logging.ERROR,
+        "CRITICAL": logging.CRITICAL
+    }
+
     # check if there's any handlers already
     if not logger.handlers:
         # create file handler if path is provided
         if file_handler_path:
+            # make sure the directory exists
+            os.makedirs(file_handler_path, exist_ok=True)  # make sure directory exists
+
             # check if the path ends with a slash
             if file_handler_path.endswith('/'):
                 file_handler = logging.FileHandler(f"{file_handler_path}{today}.log")
 
             else:
                 file_handler = logging.FileHandler(f"{file_handler_path}/{today}.log")
-            # set level to DEBUG
-            file_handler.setLevel(logging.DEBUG)
+
+            # set level for logger. if value is not found, default to DEBUG
+            file_handler.setLevel(level_mapping.get(logger_level, logging.DEBUG))
+
             # set format
             file_handler.setFormatter(logging.Formatter(
                 "%(module)s %(asctime)s [%(levelname)s]: %(message)s", "%I:%M:%S %p"))
@@ -54,7 +73,8 @@ def initialize_logger(file_handler_path=None):
 
         # Create stream handler and set level to ERROR
         stream_handler = logging.StreamHandler()
-        stream_handler.setLevel(logging.INFO)
+        # set level for logger. if value is not found, default to DEBUG
+        stream_handler.setLevel(level_mapping.get(logger_level, logging.INFO))
         stream_handler.setFormatter(logging.Formatter(
             "%(module)s %(asctime)s [%(levelname)s]: %(message)s", "%I:%M:%S %p"))
         # add stream handler to the logger
@@ -76,10 +96,13 @@ def is_ip_address(string_to_check):
         bool: True if the string is a valid IP address, False otherwise.
     """
     ip_pattern = re.compile(r'^(\d{1,3}\.){3}\d{1,3}$')
+    logging.debug(f"IP Pattern: {ip_pattern}")
     if ip_pattern.match(string_to_check):
         parts = string_to_check.split('.')
+        logging.debug(f"IP to Check: {string_to_check} | IP Parts: {parts}")
         return all(0 <= int(part) <= 255 for part in parts)
 
+    logging.debug(f"{string_to_check} does not match the IP pattern.")
     return False
 
 
@@ -96,23 +119,52 @@ def resolve_dns(domain, dns_server=None, first_result_only=True):
     """
     # Check if the domain is a URL and extract the hostname
     if domain.startswith("http"):
+        logging.debug(f"Domain: {domain} is a URL. Extracting hostname...")
         domain = urlparse(domain).hostname
+        logging.debug(f"Extracted hostname: {domain}")
 
     # Initialize the resolver
     resolver = dns.resolver.Resolver()
+    logging.debug("Initialized DNS resolver.")
 
+    # Set the DNS server if provided
     # Set the DNS server if provided
     if dns_server:
         resolver.nameservers = [dns_server]
+        logging.debug(f"DNS server set to [{dns_server}].")
 
     try:
         answer = resolver.resolve(domain)
+        logging.debug(f"Resolved {domain} to {answer}")
+
         # return the first result only if specified
         if first_result_only:
-            return answer[0].to_text()
+            first_result = answer[0].to_text()
+            # check if the result is an IP address
+            if not is_ip_address(first_result):
+                raise ValueError(f"Resolved {domain} to {first_result} which is not an IP address.")
+
+            logging.debug(f"Returning first result only: {first_result}")
+            return first_result
 
         # otherwise return all results
-        return [rdata.to_text() for rdata in answer]
+        results = [rdata.to_text() for rdata in answer]
 
-    except (dns.resolver.NoAnswer, dns.resolver.NXDOMAIN, dns.exception.Timeout) as e:
-        return f"Error resolving DNS: {e}"
+        # check if the results are valid IP addresses
+        valid_ips = []
+        for ip in results:
+            if not is_ip_address(ip):
+                logging.error(f"Resolved {domain} to {ip} which is not an IP address. This result will be ignored.")
+                continue
+            else:
+                valid_ips.append(ip)
+
+        if valid_ips:
+            logging.debug(f"Returning valid results: {valid_ips}")
+            return valid_ips
+        else:
+            raise ValueError(f"Unable to resolve {domain} to any valid IP addresses.")
+
+    except (dns.resolver.NoAnswer, dns.resolver.NXDOMAIN, dns.exception.Timeout, ValueError) as e:
+        logging.error(f"Error resolving DNS: {e}")
+        return
